@@ -89,7 +89,10 @@ class Thiele:
             lr = len(ring)
             if not 3 < lr < 8:  # skip 3-membered and big rings
                 continue
-            sp2 = sum(sh(n) == 2 and atoms[n].atomic_number in (5, 6, 7, 8, 15, 16) for n in ring)
+            # only B C N O P S with 2-3 neighbors. detects this: C1=CC=CP12=CC=CC=C2
+            if any(atoms[n].atomic_number not in (6, 7, 8, 16, 5, 15) or len(bonds[n]) > 3 for n in ring):
+                continue
+            sp2 = sum(sh(n) == 2 for n in ring)
             if sp2 == lr:  # benzene like
                 if lr == 4:  # two bonds condensed aromatic rings
                     tetracycles.append(ring)
@@ -108,15 +111,15 @@ class Thiele:
                 except StopIteration:  # exotic, just skip
                     continue
                 an = atoms[n].atomic_number
-                if lr == 7:  # skip electron-rich 7-membered rings
+                if (c := charges[n]) == -1:
+                    if an != 6 or lr != 5:  # skip any but ferrocene
+                        continue
+                elif c:  # skip any charged
+                    continue
+                elif lr == 7:  # skip electron-rich 7-membered rings
                     if an != 5:  # not B?
                         continue
                 # below lr == 5 or 6 only
-                elif (c := charges[n]) == -1:
-                    if an != 6 or lr != 5:  # ferrocene, etc.
-                        continue
-                elif c:  # skip charged
-                    continue
                 elif an in (8, 16, 34):  # O, S, Se
                     if len(bonds[n]) != 2:  # like CS1(C)C=CC=C1
                         continue
